package agendaJSP.persistence;

import java.sql.*;
import java.util.Vector;
import agendaJSP.modele.Personne;
/**
 * Cette classe gre les intractions avec la base de donnes.
 * This file is free Software
 *
 * Note : la gestion d'erreurs est trs trs minime.
 * (c) Serge Rosmorduc
 * @author rosmord
 *
 */

public class PersonneDAO {
    
    private LoginManager loginManager;
    
    public PersonneDAO(LoginManager loginManager) {
        this.loginManager= loginManager;
    }
    
    private void open() throws PersistenceException {
        loginManager.open();
    }
    
    private void close() {
        loginManager.close();
    }
    
    private Connection getConnection() {
        return loginManager.getConnection();
    }
    
    /**
     * Cre une personne dans la base.
     *
     * <p> Note sur le code : on excute deux oprations distinctes :
     * choix d'un identifiant, puis insertion dans la base. Le problme est 
     * que postgres ne permet pas de garantir leur russite, mme avec des transactions.
     *
     * En effet, mme le niveau SERIALIZABLE, le plus fort, ne permet pas d'assurer que
     * si deux clients interrogent le serveur pour recevoir un identifiant, 
     * ils recevront une valeur diffrente. 
     * 
     * Ce qui peut se passer : deux clients demandent en mme temps maxid. Ils recoivent 
     * la mme valeur. L'un des deux insre une personne, puis l'autre essaie  son 
     * tour. Comme le numro a dj t attribu, il y a erreur  l'insertion. Ce comportement
     * se produit dans *tous* les niveaux de transaction.
     *
     *En consquence de quoi, il faut considrer que create peut chouer, auquel cas, 
     *il suffit de rssayer.
     * @param p
     */
    public void create(Personne p) throws PersistenceException {
        try {
            open();
            PreparedStatement createPersonneStmt =
            getConnection().prepareStatement("insert into personne values(?,?,?)");
            createPersonneStmt.setInt(1, getMaxPersonneID() + 1);
            createPersonneStmt.setString(2, p.getNom());
            createPersonneStmt.setString(3, p.getPrenom());
            createPersonneStmt.executeUpdate();
            createPersonneStmt.close();
        }
        catch (SQLException e) {
            throw new PersistenceException(e.getMessage());
        } finally {
            close();
        }
    }
    
    /**
     * met  jour une personne dans la base
     * @param p
     */
    public void update(Personne p) throws PersistenceException {
        try {
            open();
            PreparedStatement updatePersonneStmt =
            getConnection().prepareStatement(
            "update personne set nom= ?, prenom= ? where idPersonne = ?");
            updatePersonneStmt.setInt(3, p.getIdPersonne());
            updatePersonneStmt.setString(1, p.getNom());
            updatePersonneStmt.setString(2, p.getPrenom());
            updatePersonneStmt.executeUpdate();
            updatePersonneStmt.close();
        } catch (SQLException e) {
            throw new PersistenceException(e.getMessage());
        } finally {
            close();
        }
    }
    
    /**
     * Retourne le plus grand numro de personne disponible.
     * Actuellement Mthode auxiliaire utilise par la cration.
     * Method getMaxPersonneID.
     * @return int
     */
    
    private int getMaxPersonneID() throws PersistenceException {
        int result = 0;
        try {
            PreparedStatement getMaxPersonneIDStmt =
            getConnection().prepareStatement("select max(idPersonne) from personne");
            
            
            ResultSet rs = getMaxPersonneIDStmt.executeQuery();
            
            if (rs.next()) {
                result = rs.getInt(1);
            }
            getMaxPersonneIDStmt.close();
        } catch (SQLException e) {
            throw new PersistenceException(e.getMessage());
        }
        return result;
    }
    
    public void detruirePersonne(Personne p) throws PersistenceException {
        try {
            open();
            PreparedStatement detruirePersonneParIDStmt=
            getConnection().prepareStatement("delete from Personne where idPersonne= ?");
            detruirePersonneParIDStmt.setInt(1, p.getIdPersonne());
            detruirePersonneParIDStmt.executeUpdate();
            detruirePersonneParIDStmt.close();
        } catch (SQLException e) {
            throw new PersistenceException(e.getMessage());
        } finally {
            close();
        }
    }
    
    public Vector getPersonnesParInitiales(String debut) throws PersistenceException {
        Vector v= new Vector();
        ResultSet r = null;
        String aChercher = debut + "%";
        try {
            open();
            PreparedStatement getPersonnesParInitialesStmt=
            getConnection().prepareStatement("select * from Personne where nom LIKE ?");
            getPersonnesParInitialesStmt.setString(1, aChercher);
            r = getPersonnesParInitialesStmt.executeQuery();
            while(r.next()) {
                Personne p;
                p= new Personne(r.getInt("IdPersonne"),
                r.getString("nom"), r.getString("prenom"));
                v.add(p);
            }
            getPersonnesParInitialesStmt.close();
        } catch (SQLException e) {
            throw new PersistenceException(e.getMessage());
        } finally {
            close();
        }
        return v;
    }
    
    /**
     * @param n
     * @return
     */
    public Personne getPersonneParId(int n) throws PersistenceException {
        Personne p= null;
        try {
            open();
            PreparedStatement selectPersonneParIDStmt;
            selectPersonneParIDStmt= getConnection().prepareStatement(
            "select * from Personne where idPersonne = ?");
            selectPersonneParIDStmt.setInt(1,n);
            ResultSet r;
            r= selectPersonneParIDStmt.executeQuery();
            if (r.next()) {
                
                p= new Personne(r.getInt("IdPersonne"),
                r.getString("nom"), r.getString("prenom"));
            }
            r.close();
            selectPersonneParIDStmt.close();
        } catch (SQLException e) {
            throw new PersistenceException(e.getMessage());
        }
        finally {
            close();
        }
        return p;
    }
    
    /** Getter for property loginManager.
     * @return Value of property loginManager.
     *
     */
    public agendaJSP.persistence.LoginManager getLoginManager() {
        return loginManager;
    }
    
    /** Setter for property loginManager.
     * @param loginManager New value of property loginManager.
     *
     */
    public void setLoginManager(agendaJSP.persistence.LoginManager loginManager) {
        this.loginManager = loginManager;
    }
    
}
